--
-- Nsyn.hs, the top level module of the nucleosynthesis program.
--


module Main where


import Data.Packed.Vector
import Text.Printf

import Initialize
import Integrate
import Matrices
import Reactions (ams)



-- A utility function to repeatedly apply f to the argument v
-- as long as the predicate p is satisfied and construct a list of the results.
--
iterateWhile :: (a -> Bool) -> (a -> a) -> a -> [a]
iterateWhile p f v = if p v then v : iterateWhile p f (f v) else []


-- Utility function to convert a vector of abundances by number into a vector of
-- abundances by mass.
--
numberToMass :: Vector Double -> Vector Double
numberToMass ys = zipVector (*) ams ys


-- Format the output into a pleasant to read table
--
formatOutput :: (Double, Double, Double, Vector Double) -> String
formatOutput (t, tl, h, ys) =
        let
                time       = printf "%10.2f    " t
                thermal    = concatMap (printf "%10.04e  ") (toList (subVector 0 3 ys))
                abundances = concatMap (printf "%11.04e  ") (toList (numberToMass (subVector 3 nNuclides ys)))
        in
                time ++ thermal ++ abundances


-- Output in the classical style of new123: mass fraction for p and 4He, ratio
-- to the number of protons for everyone else.
--
formatOutput' :: (Double, Double, Double, Vector Double) -> String
formatOutput' (t, tl, h, ys) =
        let
                xp   = (ys @> 4) * (ams @> 1)
                x4He = (ys @> 8) * (ams @> 5)

                yp   =  ys @> 4

                rnp   = (ys @> 3)  / yp
                rdp   = (ys @> 5)  / yp
                rtp   = (ys @> 6)  / yp
                r3Hep = (ys @> 7)  / yp
                r6Lip = (ys @> 9)  / yp
                r7Lip = (ys @> 10) / yp
                r7Bep = (ys @> 11) / yp

                metals = (sumVector (subVector 12 17 ys)) / yp

                time   = printf "%11.3f " t
                temp   = printf "%5.2f  " (ys @> 0)
                abundances = printf "%10.3e  %10.3e  %10.3e  %10.3e  %10.3e  %10.3e  %10.3e  %10.3e  %10.3e  %10.3e" rnp xp rdp rtp r3Hep x4He r6Lip r7Lip r7Bep metals
        in
                time ++ temp ++ abundances


minTemp = 1.0e-02


main = do
        let
                ys = initialConditions

                t = 1.08169e-02   -- This is just a hack: fixme!
                h = 1.0e-7
                    
                tend = 1.0e+7
                result = iterateWhile (\(t, tl, h, y) -> t < tend && ((y @> 0) > minTemp)) nsynStep (t, tend, h, ys)

        putStr $ unlines $ map formatOutput' result
